/*

atmega328 Source Code for MouseHack Interface
gaudi.ch

 

 */


/******************************************************************************
 * Definitions
 ******************************************************************************/



#define FOSC 16000000 // Clock Speed 

#define BAUD 57600 

#define MYUBRR FOSC/16/BAUD-1 

#define F_CPU 16000000UL



#define Delta_Y				0x04
#define Delta_X				0x03
#define Motion_Status		0x02

#define Mask_Motion			0x80
#define Mask_DYOVF			0x10
#define Mask_DXOVF			0x08





/******************************************************************************
 * Includes
 ******************************************************************************/




#include <avr/io.h> 

#include <stdio.h> 

#include <compat/ina90.h>

#include <stdlib.h>

#include <util/delay.h>

#include <avr/interrupt.h>

#include <avr/pgmspace.h>





/******************************************************************************
 * Table
 ******************************************************************************/






prog_uint8_t PhaseArray[70] PROGMEM =

{
0x2E, 0x27, 0x60, 0x2C, 0x5E, 0x3A, 0x22, 0x3B, 0x7E, 0x2D, 0x5F, 0x2B, 0x3C, 0x3E, 0x69, 0x21,
0x6C, 0x49, 0x3F, 0x2F, 0x5C, 0x7C, 0x28, 0x29, 0x31, 0x7B, 0x7D, 0x5B, 0x5D, 0x72, 0x63, 0x76,
0x75, 0x6E, 0x78, 0x7A, 0x6A, 0x66, 0x74, 0x4C, 0x43, 0x4A, 0x55, 0x59, 0x58, 0x5A, 0x4F, 0x30,
0x51, 0x6F, 0x61, 0x68, 0x6B, 0x62, 0x64, 0x70, 0x71, 0x77, 0x6D, 0x2A, 0x57, 0x4D, 0x42, 0x38,
0x26, 0x25, 0x24, 0x23, 0x40

};








/******************************************************************************
 * Variables
 ******************************************************************************/


int  main(void); 			



void init(unsigned int); 				// UART init

char get_char(void); 					// UART get

int  put_char(char c, FILE *stream);	// UART put

uint8_t readRegister(uint8_t address); 

char in;								// UART char



uint8_t OptiMouseReadRegister(uint8_t address);	// Mouse Read

void OptiMouseWriteRegister(uint8_t address, uint8_t data);

void OptiMouseBegin(void);						// Mouse init





signed char dx = 0;						// Variables for mouse x/y
signed char dy = 0;

int x;
int y;
char status = 0;						// Variables for mouse status

char ser_in;



int main(void) 

{ 

  	init(MYUBRR);                    		//initialize UART 
    fdevopen(put_char,NULL); 				//important if using stdio.h functions


	OptiMouseBegin();						// Init Mouse
 	OptiMouseWriteRegister(0x0a,9);
 
  	
 //	status = OptiMouseReadRegister(0x01); //Motion_Status


while(1){	// Main Loop

ser_in=get_char();

if (ser_in==65)
{
OptiMouseWriteRegister(0x0a,0);
OptiMouseWriteRegister(0x0a,9);

for (x=0; x<256; x++){
dx= OptiMouseReadRegister(0x0d);//Delta_X
put_char(OptiMouseReadRegister(0x0c),NULL);
};
};

if (ser_in<16)
{put_char(OptiMouseReadRegister(ser_in),NULL);
};


/*
printf("%c",12);
OptiMouseWriteRegister(0x0a,0);
OptiMouseWriteRegister(0x0a,9);
for (y=0; y<16; y++){
for (x=0; x<16; x++){

dx= OptiMouseReadRegister(0x0d);//Delta_X
dy= OptiMouseReadRegister(0x0c);//Delta_Y
	
printf("%c%c",pgm_read_word(&PhaseArray[64-dy]),pgm_read_word(&PhaseArray[64-dy]));
}
printf("\n\r");
};
 */

  //	status = OptiMouseReadRegister(Motion_Status); //Motion_Status



//	dx= OptiMouseReadRegister(0x0d);//Delta_X 0d

//	dy= OptiMouseReadRegister(0x0c);//Delta_Y 0c

//	printf("Mouse: %d  dx: %d  dy: %d\n\r",status,dx,dy);

//	_delay_ms(100);



   }; //while loop

}; // main

/******************************************************************************
 * Mouse API
 ******************************************************************************/



void OptiMouseBegin(void)
{
	// Re-sync (see datasheet 5.4):
	// Toggle the SLCK line from high to low to high....

	DDRB |=(1<<DDB0);  		//Set _sclkPin as output 

	PORTB |=(1<<PORTB0);		//Set _sclkPin high 

           

	_delay_us(5);

		           

	PORTB &=~(1<<PORTB0);		//Set _sclkPin low 



	_delay_us(5);
	PORTB |=(1<<PORTB0);		//Set _sclkPin high 
	
	// Wait at least tSIWTT (0.9 second?) for the
	// OptiMouse serial transaction timer to time out:


	_delay_ms(1000);
}

uint8_t readRegister(uint8_t address) {
DDRB |=(1<<DDB1);  		//Set _sdioPin as output 

  for (uint8_t i=128; i >0 ; i >>= 1) {
    PORTB &=~(1<<PORTB0);					//Set _sclkPin low 
    if ((address & i) != 0)

			{PORTB |=(1<<PORTB1);} 			//Set _sdioPin high 
		
			else    {PORTB &=~(1<<PORTB1);}		//Set _sdioPin low
	PORTB |=(1<<PORTB0);		//Set _sclkPin high 
}

	DDRB &=~ (1<<DDB1);		   //Set _sdioPin as input 

  _delay_us(50);
    _delay_us(50);

  uint8_t res = 0;
  for (uint8_t i=128; i >0 ; i >>= 1) {
		PORTB &=~(1<<PORTB0);		//Set _sclkPin low  
   	PORTB |=(1<<PORTB0);		//Set _sclkPin high
   	
  
    if(PINB&&(1<<PORTB1))
       res|= i;
  }

  return res;
}



uint8_t OptiMouseReadRegister(uint8_t address)
{
	int i = 7;
	uint8_t r = 0;
	
	// Write the address of the register we want to read:

	DDRB |=(1<<DDB1);  		//Set _sdioPin as output 

	for (; i>=0; i--)
	{

		PORTB &=~(1<<PORTB0);					//Set _sclkPin low 

		if (address & (1 << i))

				{PORTB |=(1<<PORTB1); 			//Set _sdioPin high 

				_delay_us(10);
				}		

			else    {PORTB &=~(1<<PORTB1);		//Set _sdioPin low

				_delay_us(10);
				}	
		PORTB |=(1<<PORTB0);		//Set _sclkPin high 

		_delay_us(10);
	}
	
	// Switch data line from OUTPUT to INPUT

	DDRB &=~ (1<<DDB1);		   //Set _sdioPin as input 


	// Wait a bit...
	_delay_us(50);
	_delay_us(50);

	
	// Fetch the data!
	for (i=7; i>=0; i--)
	{                 

		PORTB &=~(1<<PORTB0);		//Set _sclkPin low  

		_delay_us(10);           

		PORTB |=(1<<PORTB0);		//Set _sclkPin high

		_delay_us(10); 

		r |= (((PINB>>PORTB1)&0x01)<<i);
	}
	//_delay_us(50);
		_delay_us(20);
	return r;
}

void OptiMouseWriteRegister(uint8_t address, uint8_t data)
{
	int i = 7;
	
	// Set MSB high, to indicate write operation:
	address |= 0x80;
	
	// Write the address:


	DDRB |=(1<<DDB1);  		//Set _sdioPin as output 
	for (; i>=0; i--)
	{
		PORTB &=~(1<<PORTB0);		//Set _sclkPin low  

		if (address & (1 << i))

					{PORTB |=(1<<PORTB1);		//Set _sdioPin high 

					_delay_us(5);}   		

			else    {PORTB &=~(1<<PORTB1);		//Set _sdioPin low
					_delay_us(5);};

		
		PORTB |=(1<<PORTB0);		//Set _sclkPin high 

		_delay_us(5);
	}
	
	// Write the data:
	for (i=7; i>=0; i--)
	{
		PORTB &=~(1<<PORTB0);		//Set _sclkPin low  

		if (data & (1 << i))

				{PORTB |=(1<<PORTB1);		//Set _sdioPin high 

				_delay_us(5);}

			else    {PORTB &=~(1<<PORTB1);		//Set _sdioPin low
					_delay_us(5);}
		PORTB |=(1<<PORTB0);		//Set _sclkPin high 

		_delay_us(5);
	}
}




/******************************************************************************
 * UART API
 ******************************************************************************/


void init(unsigned int ubrr) 

{

    //***** UART init 

    asm("cli"); 

    UCSR0B = 0x00;  //Temp disable uart  

    UCSR0A = 0x00; 

    /* Set frame format: 8data, 1stop bit */ 

	UCSR0C = (3<<UCSZ00); 

	UBRR0H = (unsigned char)(ubrr>>8); 

	UBRR0L = (unsigned char)ubrr; 



    UCSR0B = (1<<RXEN0)|(1<<TXEN0);  //Enable uart (no irq, tx/rx enabled) 

    asm("sei"); 

} 

//************************************************** 

// put_char is also used by standard 

// output devices (printf,putchar) 

//************************************************** 

int put_char(char c, FILE *stream)

{ 

    while((UCSR0A&(1<<UDRE0)) !=(1<<UDRE0)){;} 

    UDR0 = c; 

    return 0; 

} 

//************************************************** 

char get_char(void) 

{ 

    while((UCSR0A&(1<<RXC0)) != (1<<RXC0)){;} 

    return UDR0; 

} 

